library(shiny)

library(data.table) #ITime
library(lubridate) #USAR ESSE PARA ManipulaÃ§Ã£o de datas (funcao floor_date e conversao de compo impordadas de csv com separadores de ";")
library(leaflet)    #https://rstudio.github.io/leaflet/showhide.html

#====================================
#Para publicar

#install.packages('rsconnect')
#https://www.shinyapps.io/admin/#/dashboard

# rsconnect::setAccountInfo(name='mariotto',    #Instalar registro (única vez no R)
# 			  token='1FA7E69DA2031906E0CB99E643CA6CC7',
# 			  secret='<SECRET>') 

# library(rsconnect)
#rsconnect::deployApp('path/to/your/app') #Ativar rsconnect

# library(rsconnect)
# deployApp() #Publicar App

#===========================================
#library(plotly) #Plot 3d
#plot_ly(x=x,y=y,z=z, type="surface")

#PREPARACAO INICIAL DOS DADOS ----
dfPonto <- read.csv(file="TabGrandFisicas.csv",header=T)
dfPonto$Data = strptime(dfPonto$Data, "%Y-%m-%d %H:%M:%S")
dfPonto$Horario=as.ITime(dfPonto$Data)
dfPonto$Semana=wday(dfPonto$Data)
dfPonto$Horario15<-as.ITime(floor_date(dfPonto$Data, "15 mins")) #Aproxima para o horÃÂ¡rio mais cedo (outra opÃÂ§ÃÂ£o round_date)
dfPonto$Horario15 <- factor(substr(dfPonto$Horario15, 1, 5))
dfPonto$Mes=month(dfPonto$Data)

TabPontosTot <- read.csv(file="circular_pontos_por_linha.csv", header=T, sep=";", dec=",")
TabPercursoLinhas <- read.csv(file="TabPercursoLinhas.csv",header=T)


#Tabela de tradução de codigo e descricao abreviada
TabLocalAbreviatura=TabPontosTot[!duplicated(TabPontosTot$codigo.local),][,
                                  c("codigo.local","abreviatura","latitude","longitude")]
TabLocalAbreviatura=TabLocalAbreviatura[order(TabLocalAbreviatura$codigo.local),]
TabLocalAbreviatura$Idx=c(1:nrow(TabLocalAbreviatura))
names(TabLocalAbreviatura)[1] <- "x" #Renomeia compo "código" por "x"
names(TabLocalAbreviatura)[2] <- "nome"

TabCodAbreviatura=TabPontosTot[!duplicated(TabPontosTot$codigo),][,
                                  c("codigo","abreviatura","latitude","longitude")]
TabCodAbreviatura=TabCodAbreviatura[order(TabCodAbreviatura$codigo),]
TabCodAbreviatura$Idx=c(1:nrow(TabCodAbreviatura))
names(TabCodAbreviatura)[1] <- "x" #Renomeia compo "código" por "x"
names(TabCodAbreviatura)[2] <- "nome"

TabPontosNome=list() #Nomes dos pontos para a seleção de ponto (filtro)
for (i in 1:nrow(TabCodAbreviatura)) {
  TabPontosNome[i]=i
}
names(TabPontosNome)=TabCodAbreviatura$nome
i=length(TabPontosNome)+1
TabPontosNome=append(TabPontosNome, list(Todos=i))

TabHorarios=dfPonto[!duplicated(dfPonto$Horario15),][,c("Horario15","Horario15")]
TabHorarios=TabHorarios[order(TabHorarios$Horario15),]
TabHorarios$Idx=c(1:nrow(TabHorarios))
names(TabHorarios)[1] <- "x"
names(TabHorarios)[2] <- "nome"

minT = min(dfPonto$Horario)
maxT = max(dfPonto$Horario)

#================================
# Definições do mapa leaflet
mapa <- leaflet() %>% addTiles()  #mapa <- leaflet(); mapa <- addTiles(mapa)

for(iLinha in 1:4) {
  #Seleciona o numero da linha para analise
  TabRastreamento=subset(TabPercursoLinhas, id_linha==iLinha)

  if (iLinha ==1) {
    color = "#03F"
    sgroup = "Linha 1"
  }
  else if (iLinha ==2) {
    color = "red"
    sgroup = "Linha 2"
  }
  else if (iLinha ==3) {
    color = "black"
    sgroup = "Linha 3"
  }
  else {
    color = "green"
    sgroup = "Linha 4"
  }

  mapa %<>% addCircleMarkers(lng=TabCodAbreviatura$longitude,
                    lat=TabCodAbreviatura$latitude,
                    #popup=TabCodAbreviatura$nome,
                    weight=2,
                    color = "black",
                    radius = 3,
                    group = "Pontos",
                    label = paste(TabCodAbreviatura$x, "-", TabCodAbreviatura$nome) ) %>%

  addPolylines(lng=TabRastreamento$longitude,
                    lat=TabRastreamento$latitude,
                    weight=3,
                    color = color,
                    group = sgroup,
                    popup=sgroup)
}





#=========================================================

#-----------------------------------
GrafDispersao <- function (iIni, dfSubset) {
  n=nrow(TabCodAbreviatura) #$codigo)
  op <- par(xpd = TRUE, mfrow=c(2,3))#, oma = c(4,1,1,1))
  
  iGraf=0
  coordMedianas <- c()
  
  for (i in 1:n) {
    nCod = TabCodAbreviatura$x[i]
    #dfSubsetPonto <- subset(dfSubset, NumPonto==unique(subset(TabSubPontos,codigo==i)$NumPontos))
    dfSubsetPonto <- dfSubset[[i]]
    if (nrow(dfSubsetPonto) > 0) {
      iGraf = iGraf + 1
      if (iGraf >= iIni) {
        #Pontos dispersão
        plot (dfSubsetPonto$Longitude ~ dfSubsetPonto$Latitude, xlab="Latitude", ylab="Longitude",
              pch=46, col="black", xlim=rev(range(dfSubsetPonto$Latitude)) ) #ou pch=46
        
        #Localização do ponto
        points(TabCodAbreviatura$longitude[i] ~ TabCodAbreviatura$latitude[i],
               pch=15, col="red", cex=1.4)
        
        LatMedian = median(dfSubsetPonto$Latitude)
        LonMedian = median(dfSubsetPonto$Longitude)
        coordMedianas$latitude[i] = LatMedian
        coordMedianas$longitude[i] = LonMedian
        
        #Pontos mediana
        points(LonMedian ~ LatMedian, pch=17, col="green", cex=1.4)
        title(paste("Cod", nCod, "-", TabCodAbreviatura$nome[i]))
        
        # print(paste("Cod. Ponto=", nCod, "-", TabCodAbreviatura$nome[i],
        #             ", Longitude=",  LonMedian,
        #             ", Latitude=", LatMedian,
        #            sep ="") )
      
        if (iGraf  >= iIni + 5) break #Plota 6 gráficos e termina
      }
    }
      #else print (paste("Cod. Ponto=", i, " não posssui paradas registradas", sep=""))
  }
  par(op)

}


"===================================================="
# Define UI - interface usuário ----
CirculinoUI <- fluidPage(

  titlePanel("Painel Circulino!"),
  
  sidebarLayout(
    sidebarPanel( 
      fluidRow (
        column(5,
          radioButtons(inputId = "ResultAnaliseX",
                      label = "Avaliação:",
                      choices = list("Ponto" = 1,
                                     "Local" = 2,
                                     "Horário" = 3), 
                      selected = 2
          )
        ),
        column(7,
          radioButtons(inputId = "ResultAnaliseY",
                      label = "Indicador:",
                      choices = list("Aceleração" = 1, 
                                     "Distância" = 2,
                                     "Tempo parada" = 3,
                                     "Veloc parada" = 4,
                                     "Num. paradas" = 5), 
                      selected = 3
          )
        )
      ),
      fluidRow (
        column(6,
          radioButtons(inputId = "ResultTipo",
                      label = "Técnica:",
                      choices = list("Variação" = 1, 
                                     "Média" = 2,
                                     "Soma" = 3), 
                      selected = 3
          )
        ),
        column(6,
          radioButtons(inputId = "ResultPeriodico",
                      label = "Periodicidade:",
                      choices = list("Mensal" = 1, 
                                     "Semanal" = 2), 
                      selected = 1
          )
        )
      ),
      fluidRow (
        column(6,
          checkboxGroupInput(inputId = "LinhaList",
                   label = "Linhas:",
                   choices = list("Linha 1" = 1, 
                                   "Linha 2" = 2, 
                                   "Linha 3" = 3,
                                   "Linha 4" = 4),
                   selected = c(1,2,3,4)
          ),
          selectInput(inputId ="PontoList", label = "Ponto de ônibus:", #h3("Select box"), 
                       choices = TabPontosNome, selected = length(TabPontosNome))
        ),
        column(6,
          checkboxGroupInput(inputId = "CirculinoList",
                    label = "Ônibus:",
                    choices = list("Circulino 1" = 1, 
                                   "Circulino 2" = 2, 
                                   "Circulino 3" = 3,
                                   "Circulino 4" = 4,
                                   "Circulino 5" = 5,
                                   "Circulino 6" = 6,
                                   "Circulino 7" = 7,
                                   "Circulino 8" = 8) ,
                    selected = c(1,2,3,4,5,6,7,8)
          )
        )
      ),
      fluidRow (
        dateRangeInput(inputId = "DataRange",
                    label = "Periodo análise:",
                    start= min(dfPonto$Data),
                    end= strptime("2019-01-01", "%Y-%m-%d"), #max(dfPonto$Data),
                    min = min(dfPonto$Data),
                    max = as.IDate(max(dfPonto$Data))+1,
                    language = "pt-BR",
                    format = "dd/mm/yy"
        ),
        column(6,
          textInput(inputId = "HoraRangeI",
                    label = "Inicial:",
                    value=substr(as.ITime(minT), 1, 5), 
                    width = "80%"
          )
        ),
        
        column(6,
          textInput(inputId = "HoraRangeF",
                    label = "Final:",
                    value=substr(as.ITime(maxT)+as.ITime("00:01"), 1, 5),
                    width = "80%"
          )
        ),
        
        checkboxGroupInput(inputId = "MesList",
                   label = "Meses:",
                   inline = TRUE,
                   # choices = list("Jan" = 1, 
                   #                "Fev" = 2, 
                   #                "Mar" = 3,
                   #                "Abr" = 4,
                   #                "Mai" = 5,
                   #                "Jun" = 6,
                   #                "Jul" = 7,
                   #                "Ago" = 8,
                   #                "Set" = 9,
                   #                "Out" = 10,
                   #                "Nov" = 11,
                   #                "Dez" = 12),
                   choices = list("1" = 1, 
                                  "2" = 2, 
                                  "3" = 3,
                                  "4" = 4,
                                  "5" = 5,
                                  "6" = 6,
                                  "7" = 7,
                                  "8" = 8,
                                  "9" = 9,
                                  "10" = 10,
                                  "11" = 11,
                                  "12" = 12),
                   selected = 1:12
        ),
        checkboxGroupInput(inputId = "SemList",
                   label = "Dias da semana:",
                   inline = TRUE,
                    choices = list("Dom" = 1, 
                                  "Seg" = 2, 
                                  "Ter" = 3,
                                  "Qua" = 4,
                                  "Qui" = 5,
                                  "Sex" = 6,
                                  "Sab" = 7),
                   selected = 1:7
        )
      )

    ), #sidebarPanel
    
    # Main panel for displaying outputs ----
    mainPanel(
      tabsetPanel(
        tabPanel("Principal",  
          plotOutput(outputId = "GrafGrandezas"),
          plotOutput(outputId = "GrafGrandezasPer")
        ),
        tabPanel("Histograma",
          fluidRow (
            column(6,
              sliderInput(inputId = "AcelRange",
                          label = "Faixa de aceleração:",
                          min = 0,
                          max = 8,
                          step= 0.1,
                          value = c(0.1,3))
            ),
            column(6,              
              sliderInput(inputId = "TempoRange",
                          label = "Faixa de tempos de parada:",
                          min = 0,
                          max = 60,
                          step= 1,
                          value = c(0,20))
            )
          ),
          fluidRow (
            column(6,
              sliderInput(inputId = "DistRange",
                          label = "Faixa de Distância:",
                          min = floor(min(dfPonto$DistParada)),
                          max = ceiling(max(dfPonto$DistParada)),
                          step= 1,
                          value = c(floor(min(dfPonto$DistParada)),ceiling(max(dfPonto$DistParada))))
            ),
            column(6,
              sliderInput(inputId = "VelRange",
                          label = "Faixa de velocidade:",
                          min = 0,
                          max = ceiling(max(dfPonto$VelParada)),
                          step= 1,
                          value = c(0,ceiling(max(dfPonto$VelParada))))
            ),
            sliderInput(inputId = "bins",
                        label = "Segmentos histograma:",
                        min = 1,
                        max = 100,
                        value = 30)
    
          ),
          plotOutput(outputId = "GrafDistrib")
        ),
        tabPanel("Mapa",leafletOutput(outputId = "GrafMapa", height = 600) ),
        
        tabPanel("Disp. Pontos",
                 h4("Dispersão das paradas na proximidade do ponto"),
                 h5("Legenda: Vermelho= Coordenadas do ponto, Verde=Coordenadas da mediana"),
                 plotOutput(outputId = "GrafDisper1"),
                 plotOutput(outputId = "GrafDisper7"),
                 plotOutput(outputId = "GrafDisper13"),
                 plotOutput(outputId = "GrafDisper19"),
                 plotOutput(outputId = "GrafDisper25"),
                 plotOutput(outputId = "GrafDisper31"),
                 plotOutput(outputId = "GrafDisper37"),
                 plotOutput(outputId = "GrafDisper43"),
                 plotOutput(outputId = "GrafDisper49")
        ),
        tabPanel("Disp. Acel/Vel", plotOutput(outputId = "GrafDispAcelVel") ),
        tabPanel("Interdep.",
                 plotOutput(outputId = "GrafIntLinPon"),
                 plotOutput(outputId = "GrafIntLinCir"),
                 plotOutput(outputId = "GrafIntCirMes") )
      )
    )
  ) #sidebarLayout
) 

#===================================================================================
#SERVER ----
CirculinoServer <- function(input, output) {

  # Histogram of the Old Faithful Geyser Data 
  # with requested number of bins
  # This expression that generates a histogram is wrapped in a call
  # to renderPlot to indicate that:
  #
  # 1. It is "reactive" and therefore should be automatically
  #    re-executed when inputs (input$bins) change
  # 2. Its output type is a plot


  dfSubsetData <- reactive({
    MinData <- strptime(input$DataRange[1], "%Y-%m-%d")
    MaxData <- strptime(input$DataRange[2]+1, "%Y-%m-%d")
    subset(dfPonto, Data<=MaxData & Data >= MinData)
  })
  dfSubsetAcel <- reactive({ #FILTROS ----
    subset(dfSubsetData(), Aceleracao >= input$AcelRange[1]  & Aceleracao <= input$AcelRange[2] &
                          TempoParada >= input$TempoRange[1] & TempoParada <= input$TempoRange[2] &
                          DistParada >= input$DistRange[1] & DistParada <= input$DistRange[2] &
                          VelParada >= input$VelRange[1] & VelParada <= input$VelRange[2])
  })
  dfSubsetHora <- reactive({
    MinHora <- as.ITime(input$HoraRangeI)
    MaxHora <- as.ITime(input$HoraRangeF)
    if(!is.na(MinHora) && !is.na(MaxHora) && MinHora < MaxHora && MinHora > minT && MaxHora < maxT)
      subset(dfSubsetAcel(), Horario<=MaxHora & Horario >= MinHora)
    else
      dfSubsetAcel()
  })

  # dfSubsetMes <- reactive({
  #   MinMes <- input$MesRange[1]
  #   MaxMes <- input$MesRange[2]
  #   subset(dfSubsetHora(), Mes<=MaxMes & Mes >= MinMes)
  # })
  dfSubsetMes <- reactive({
    if (length(input$MesList)>=12 && length(input$SemList)>=7)
      dfSubsetHora() #Sem filtragem
    else
      subset(dfSubsetHora(), Mes %in% input$MesList & Semana %in% input$SemList)
  })

  dfSubsetCir <- reactive({
    if (length(input$CirculinoList)>=8)
      dfSubsetMes() #Sem filtragem
    else
      subset(dfSubsetMes(), Circulino %in% input$CirculinoList)
  })

  dfSubsetPon <- reactive({
    i = as.numeric(input$PontoList)
    if (i==length(TabPontosNome))
      dfSubsetCir() #Sem filtragem
    else
      subset(dfSubsetCir(), CodPonto == TabCodAbreviatura$x[i])
  })
  
  dfSubset <- reactive({
    if (length(input$LinhaList)>=4)
      dfSubsetPon() #Sem filtragem
    else
      subset(dfSubsetPon(), Linha %in% as.numeric(input$LinhaList))
  })
  
  
  dfSubsetCod <- reactive({
    df <- list()
    n=nrow(TabCodAbreviatura) #$codigo
    for (i in 1:n) {
      nCod = TabCodAbreviatura$x[i]
      df[[i]] <- subset(dfSubset(), CodPonto==nCod)
    }
    df
  })
  
#===========================================
  output$GrafDistrib <- renderPlot( { #Histograma ----
    if (input$ResultAnaliseY == 1) {
      xNome = "Aceleração"
      xNomeUnidade = "(m/s2)"
      x <- dfSubset()$Aceleracao
      xMin = 0
    }
    else if (input$ResultAnaliseY == 2) {
      xNome = "Distancia do ponto"
      xNomeUnidade = "(m)"
      x <- dfSubset()$DistParada
      xMin = min(x)
    }
    else if (input$ResultAnaliseY == 3) {
      xNome = "Tempo de parada"
      xNomeUnidade = "(s)"
      x <- dfSubset()$TempoParada
      xMin = 0
    }
    else {
      xNome = "Velocidade de parada"
      x <- dfSubset()$VelParada
      xNomeUnidade = "(m/s)"
      xMin = 0
    }
    bins <- seq(min(x), max(x), length.out = input$bins + 1)

    hist(x, breaks = bins, col = "#75AADB", border = "white",
         xlab = paste(xNome, xNomeUnidade),
         ylab = "Frequencia",
         main = paste("Histograma de ", xNome, "nos pontos"),
         xlim=c(xMin,max(x)))
    })

  
  #===========================================
  output$GrafDisper1 <- renderPlot( { #Dispersão Pontos ----
    GrafDispersao (1,dfSubsetCod())
  })
  output$GrafDisper7 <- renderPlot( { 
    GrafDispersao (7,dfSubsetCod())
  })
  output$GrafDisper13 <- renderPlot( { 
    GrafDispersao (13,dfSubsetCod())
  })
  output$GrafDisper19 <- renderPlot( { 
    GrafDispersao (19,dfSubsetCod())
  })
  output$GrafDisper25 <- renderPlot( { 
    GrafDispersao (25,dfSubsetCod())
  })
  output$GrafDisper31 <- renderPlot( { 
    GrafDispersao (31,dfSubsetCod())
  })
  output$GrafDisper37 <- renderPlot( { 
    GrafDispersao (37,dfSubsetCod())
  })
  output$GrafDisper43 <- renderPlot( { 
    GrafDispersao (43,dfSubsetCod())
  })
  output$GrafDisper49 <- renderPlot( { 
    GrafDispersao (49,dfSubsetCod())
  })
  
  output$GrafDispAcelVel <- renderPlot( { #Dispersão Acel x Vel ----
    #Dispersão Aceleração x Velocidade
    
    #print("GrafDispAcelVel")
    # TestePch <- c()
    # for (i in 1:nrow(df)) {
    #   if (Teste$VelParada[i] > 1.)
    #     TestePch[i]=46#183
    #   else
    #     TestePch[i]=46
    # }
    plot(dfSubset()$VelParada, dfSubset()$Aceleracao, ylab="Aceleração", xlab="Velocidade", pch=46)
  })

  output$GrafIntLinPon  <- renderPlot( {
    plot (dfSubset()$Linha ~ dfSubset()$CodPonto, ylab="Linha", xlab="Código ponto")
  }) 
  output$GrafIntLinCir <- renderPlot( { 
    plot (dfSubset()$Linha ~ dfSubset()$Circulino, ylab="Linha", xlab="Circulino")
  })
  output$GrafIntCirMes <- renderPlot( { 
    plot (dfSubset()$Circulino ~ dfSubset()$Mes, ylab="Circulino", xlab="Mês")
  })

  # Mapa ------------------------
  output$GrafMapa <- renderLeaflet ({ #Grafico Mapa ----
    #https://github.com/rstudio/shiny-examples/tree/master/086-bus-dashboard

    if (input$ResultAnaliseX == 1) {
      x <- dfSubset()$CodPonto
      xNome = "por ponto"
      TabNomes <- TabCodAbreviatura
      xCor="#03F" #Azul
    }
    else {
      x <- dfSubset()$CodLocal
      xNome = "por local"
      TabNomes <- TabLocalAbreviatura
      xCor="red"
    }

    #------------------------------ 
    if (input$ResultAnaliseY == 1) {
      yNome = "Aceleração"
      y <- dfSubset()$Aceleracao
    }
    else if (input$ResultAnaliseY == 2) {
      yNome = "Distancia"
      y <- dfSubset()$DistParada
    }
    else if (input$ResultAnaliseY == 3) {
      yNome = "Tempo"
      y <- dfSubset()$TempoParada
    }
    else if (input$ResultAnaliseY == 4) {
      yNome = "Velocidade"
      y <- dfSubset()$VelParada
    }
    else {
      yNome = "Paradas"
      y <- summary.factor(x)
    }
    
    if (input$ResultAnaliseY == 5) {
      #Num paradas
      Circulo <- data.frame(x=names(y),y=y)
    }
    else {
      if (input$ResultTipo == 3) {
        Circulo <- aggregate(y, list(x), sum, na.rm = TRUE)
        Operac = "soma"
      }
      else {
        Circulo <- aggregate(y, list(x), mean, na.rm = TRUE)
        Operac = "média"
      }
      names(Circulo)[1] <- "x"
      names(Circulo)[2] <- "y"
    }
    CirculoMax <- max(Circulo$y)
    Circulo$Tamanho <- 30. * Circulo$y / CirculoMax
    
    for (i in 1:nrow(Circulo)) {
      Circulo$longitude[i] <- TabNomes$longitude[which(TabNomes$x==Circulo$x[i])]
      Circulo$latitude [i] <- TabNomes$latitude[which(TabNomes$x==Circulo$x[i])]
      Circulo$nome     [i] <- as.character(TabNomes$nome[which(TabNomes$x==Circulo$x[i])] )
    } 
    
    if (input$ResultAnaliseY == 5)
      NomeGrupo = paste(yNome, xNome) #Número paradas
    else
      NomeGrupo = paste(yNome, Operac, xNome)
    
    #mapaIterac <- addCircleMarkers(mapa, lng=Circulo$longitude,
    addCircleMarkers(mapa, lng=Circulo$longitude,
                        lat=Circulo$latitude,
                        #popup=Circulo$nome,
                        weight=1,
                        color = xCor,
                        radius = Circulo$Tamanho,
                        group = NomeGrupo,
                        label = paste(Circulo$x, " - ",
                                      Circulo$nome, ": ",
                                      as.integer(Circulo$y / (CirculoMax/100)),"%", sep="") ) %>%
    
    #Define grupos
    #mapaIterac <- addLayersControl(mapaIterac,
    addLayersControl(
      #baseGroups = c(), 
      overlayGroups = c("Pontos", NomeGrupo, "Linha 1", "Linha 2", "Linha 3", "Linha 4"),
      options = layersControlOptions(collapsed = FALSE)
    )

    #mapaIterac
  })
  
parGraf <- reactive({ #Parametros gráficos para o painel principal ----
  parametros <- list()
  if (input$ResultAnaliseX == 1) {
    parametros$x <- dfSubset()$CodPonto
    parametros$xNome = "Ponto de parada"
    parametros$TabNomes <- TabCodAbreviatura
  }
  else if (input$ResultAnaliseX == 2) {
    parametros$x <- dfSubset()$CodLocal
    parametros$xNome = "Local"
    parametros$TabNomes <- TabLocalAbreviatura
  }
  else {
    parametros$x <- dfSubset()$Horario15
    parametros$xNome = "Horário"
    parametros$TabNomes <- TabHorarios
  }
  
  #------------------------------ 
  if (input$ResultAnaliseY == 1) {
    parametros$yNome = "Aceleração"
    parametros$yUnid = "(m/s2)"
    parametros$y <- dfSubset()$Aceleracao
    parametros$yMinGraf = 0
  }
  else if (input$ResultAnaliseY == 2) {
    parametros$yNome = "Distancia do ponto"
    parametros$yUnid = "(m)"
    y=dfSubset()$DistParada
    parametros$y <- y
    parametros$yMinGraf = min(y)
  }
  else if (input$ResultAnaliseY == 3) {
    parametros$yNome = "Tempo de parada"
    parametros$yUnid = "(s)"
    y = dfSubset()$TempoParada
    parametros$y <- y
    parametros$yMinGraf = min(y)
  }
  else if (input$ResultAnaliseY == 4) {
    parametros$yNome = "Velocidade de parada"
    parametros$yUnid = "(m/s)"
    y = dfSubset()$VelParada
    parametros$y <- y
    parametros$yMinGraf = min(y)
  }
  else {
    parametros$yNome = "Número de paradas"
    parametros$yUnid = ""
    parametros$y <- parametros$x
    # y <- summary.factor(x)
    # if (input$ResultAnaliseX == 3)
    #   x<-factor(names(y))  #Horario 
    # else
    #   x=as.numeric(names(y))
    parametros$yMinGraf = 0
  }
  parametros
})

  #===========================================
  output$GrafGrandezas <- renderPlot( { # Gráfico XY ----
 
    if (input$ResultAnaliseY != 5 || input$ResultTipo != 1) {
      # Não é simultanemente numero de paradas e variação
      x <- parGraf()$x
      y <- parGraf()$y
      TabNomes <- parGraf()$TabNomes
    
      #Sem separação mensal e com análise de quartis
      # if (input$ResultAnaliseX == 3 && input$ResultAnaliseY == 5) {
      #   #Organiza numero de paradas
      #   TesteMean = data.frame(x=parGraf()$x,y=parGraf()$y)
      #   if (input$ResultAnaliseX == 3) TesteMean=subset(TesteMean, y>0)
      #     NomeMedida = ""
      # }
      if (input$ResultAnaliseY == 5) {
        TesteMean <- aggregate(y, list(x), length) #Contagem
        NomeMedida = ""
      }
      else {
        #Calcula médias ou somas
        if (input$ResultTipo == 2 || input$ResultTipo == 1) {
          TesteMean <- aggregate(y, list(x), mean, na.rm = TRUE) #Media
          NomeMedida = "Média da"
        }
        else {
          TesteMean <- aggregate(y, list(x), sum, na.rm = TRUE) #Soma
          NomeMedida = "Soma da"
        }
      }
      names(TesteMean)[1] <- "x"
      names(TesteMean)[2] <- "y"
  
      LimiteMin = quantile(TesteMean$y)[[2]] #1o quartil
      LimiteMax = quantile(TesteMean$y)[[4]] #3o quartil
  
      iMax = 1
      iMin = 1
      iNor = 1
      sublabsMax=c()
      sublabsMin=c()
      sublabsNor=c()
      ColMax=2  #Vermelho
      ColMin=4  #Azul
      ColNor=1  #Preto
      if (input$ResultTipo == 1 ) { #|| input$ResultAnaliseY == 5) {
        #Grafico de vaiacao ou barras (BOX oU BAR)
        ColNor=8 #cinza
      }
        
      for (i in 1:nrow(TesteMean)) {
        #Avalia os quartis
        NomePonto = as.character(TabNomes$nome[which(TabNomes$x==TesteMean$x[i])] )
        if (TesteMean$y [i] < LimiteMin) {
          sublabsMin$Idx[iMin] = i
          sublabsMin$Num[iMin] = TesteMean$x[i]
          sublabsMin$Nome[iMin] = NomePonto
          TesteMean$col[i] <- ColMin
          TesteMean$pch[i] <- 2 #Triangulo
          iMin=iMin+1
        }
        else if (TesteMean$y [i] > LimiteMax) {
          sublabsMax$Idx[iMax] = i
          sublabsMax$Num[iMax] = TesteMean$x[i]
          sublabsMax$Nome[iMax] = NomePonto
          TesteMean$col[i] <- ColMax
          TesteMean$pch[i] <- 12 #Quadrado, 4=X
          iMax=iMax+1
        }
        else {
          sublabsNor$Idx[iNor] = i
          sublabsNor$Num[iNor] = TesteMean$x[i]
          sublabsNor$Nome[iNor] = NomePonto
          TesteMean$col[i] <- ColNor
          TesteMean$pch[i] <- 20
          iNor=iNor+1
        }
      }
      # if (input$ResultAnaliseY == 5) {
      #   #Quantidade de paradas
      #   barplot(y, xaxt="n", col=TesteMean$col, ylab=paste(parGraf()$yNome,parGraf()$yUnid), space=0,border="white") #ylim=c(parGraf()$yMinGraf,max(y))  
      #   
      #   TesteMean$x = as.numeric(TesteMean$x)
      #   axis(1,at=match(sublabsNor$Num,TesteMean$x)-0.5,labels=sublabsNor$Nome,las=2, adj=0.5, cex.axis=0.7, col.axis=1)
      #   axis(1,at=match(sublabsMin$Num,TesteMean$x)-0.5,labels=sublabsMin$Nome,las=2, adj=0.5, cex.axis=0.7, col.axis=ColMin)
      #   axis(1,at=match(sublabsMax$Num,TesteMean$x)-0.5,labels=sublabsMax$Nome,las=2, adj=0.5, cex.axis=0.7, col.axis=ColMax)
      #   title(paste(yNome, "por", xNome))
      # }
      #else if (input$ResultTipo == 1) {
      
      if (input$ResultTipo == 1) {
        #Resultados de variação
        x <- as.numeric(x)
        boxplot(y ~ x, xaxt="n", col=TesteMean$col,
                ylim=c(parGraf()$yMinGraf,max(y)),ylab=paste(parGraf()$yNome, parGraf()$yUnid))
        
        TesteMean$x = as.numeric(TesteMean$x)
        axis(1,at=match(sublabsNor$Num,TesteMean$x),labels=sublabsNor$Nome,las=2, adj=0, cex.axis=0.7, col.axis=1)
        axis(1,at=match(sublabsMin$Num,TesteMean$x),labels=sublabsMin$Nome,las=2, adj=0, cex.axis=0.7, col.axis=ColMin)
        axis(1,at=match(sublabsMax$Num,TesteMean$x),labels=sublabsMax$Nome,las=2, adj=0, cex.axis=0.7, col.axis=ColMax)
        title(paste("Variação da ", parGraf()$yNome, "por", parGraf()$xNome))
      }
      else {
        #Resultados de média ou soma
        #barplot(TesteMean$y, col=TesteMean$col, xaxt="n", las=2, adj=1, cex.axis=0.7)
        #barplot(TesteMean[,c("y","x")], xaxt="n", col=TesteMean$col)
  
        #if (input$ResultAnaliseX == 3)
          TesteMean$Idx=c(1:nrow(TesteMean)) #Horas no eixo X
        # else {
        #   for (i in 1:nrow(TesteMean)) {
          #Cria Idx em funcao do codigo do ponto
            # TesteMean$Idx[i] = iTabNomes$Idx[which(TabNomes$x==TesteMean$x[i])]
        #   }
        # }
        plot(TesteMean$y ~ TesteMean$Idx, type="p", pch=TesteMean$pch, xaxt="n",
             ylab=paste(parGraf()$yNome, parGraf()$yUnid), xlab="", col=TesteMean$col)
        lines(TesteMean$y ~ TesteMean$Idx)
    
        axis(1,at=match(sublabsNor$Idx,TesteMean$Idx),labels=sublabsNor$Nome,las=2, adj=0, cex.axis=0.7, col.axis=1)
        axis(1,at=match(sublabsMin$Idx,TesteMean$Idx),labels=sublabsMin$Nome,las=2, adj=0, cex.axis=0.7, col.axis=ColMin)
        axis(1,at=match(sublabsMax$Idx,TesteMean$Idx),labels=sublabsMax$Nome,las=2, adj=0, cex.axis=0.7, col.axis=ColMax)
        title(paste(NomeMedida, parGraf()$yNome, "por", parGraf()$xNome))
      }
    }
    
  }) #output$GrafGrandezas

  #===========================================
  output$GrafGrandezasPer <- renderPlot( { # Gráfico XY mensal ou semanal ----
    x <- parGraf()$x
    y <- parGraf()$y
    TabNomes <- parGraf()$TabNomes
    
    #------------------------------ 
    if (input$ResultPeriodico == 1) {
      #Mensal
      x2=dfSubset()$Mes
      n =12
      NomePer="Mês"
    }
    else {
      #Semanal
      x2 <- dfSubset()$Semana
      n=7
      NomePer="Dia semana"
    }
    
    #Resultados mensais
    if (input$ResultAnaliseY == 5) {
      #Organiza numero de paradas
      # TesteMean = data.frame(x=x,y=y)
      # if (input$ResultAnaliseX == 3) TesteMean=subset(TesteMean, y>0)
      TesteMean <- aggregate(y, list(x,x2), length) #Contagem
      NomeMedida = ""
    }
    else {
      if (input$ResultTipo != 3) {
        TesteMean <- aggregate(y, list(x,x2), mean, na.rm = TRUE) #média
        NomeMedida = "Média da"
      }
      else {
        TesteMean <- aggregate(y, list(x,x2), sum, na.rm = TRUE) #soma
        NomeMedida = "Soma da"
      }
    }#
    names(TesteMean)[1] <- "x"
    names(TesteMean)[2] <- "Mes"
    names(TesteMean)[3] <- "y"
    
    for (i in 1:nrow(TesteMean)) {
      #Cria Idx em funcao do codigo do ponto
      TesteMean$Idx[i] = TabNomes$Idx[which(TabNomes$x==TesteMean$x[i])]
    }
    plot(TesteMean$y ~ TesteMean$Idx, type="n", xaxt="n",
                            ylab=paste(parGraf()$yNome, parGraf()$yUnid), xlab="")
    
    for (i in 1:n)
    {
      Teste <- subset(TesteMean,  Mes==i )
      lines(Teste$Idx, Teste$y, col=i)
      points(Teste$Idx, Teste$y, col=i, pch=i) #pch=149)
    }
    legend("topright", legend=c(1:n), text.col=c(1:n), horiz=TRUE, cex=0.7, title = NomePer,
           pch=c(1:n), col=c(1:n))
    axis(1, at=TabNomes$Idx,labels=TabNomes$nome, las=2, adj=0, cex.axis=0.7)
    title(paste(NomeMedida, parGraf()$yNome, "por", parGraf()$xNome) )

  }) #output$GrafGrandezasPer

  
} #CirculinoServer

#============================================== Main ----

shinyApp(ui = CirculinoUI, server = CirculinoServer)

